@@ -1,4 +1,4 @@
-// Copyright (C) 2010-2013 Luis Pedro Coelho <luis@luispedro.org>
+// Copyright (C) 2010-2014 Luis Pedro Coelho <luis@luispedro.org>
 //
 // License: MIT (Check COPYING file)
 
@@ -550,8 +550,54 @@ PyObject* py_rank_filter(PyObject* self, PyObject* args) {
     return PyArray_Return(output);
 }
 
+template<typename T>
+void mean_filter(numpy::aligned_array<double> res, const numpy::aligned_array<T> array, const numpy::aligned_array<T> Bc, const int mode, const double cval) {
+    gil_release nogil;
+    const int N = res.size();
+    typename numpy::aligned_array<T>::const_iterator iter = array.begin();
+    filter_iterator<T> fiter(array.raw_array(), Bc.raw_array(), ExtendMode(mode), true);
+    const int N2 = fiter.size();
+    double* rpos = res.data();
+
+    for (int i = 0; i != N; ++i, ++rpos, fiter.iterate_both(iter)) {
+        int n = N2;
+        double sum = 0;
+        for (int j = 0; j != N2; ++j) {
+            T val;
+            if (fiter.retrieve(iter, j, val)) sum += val;
+            else if (mode == ExtendConstant) sum += cval;
+            else --n;
+        }
+        *rpos = sum/n;
+    }
+}
+PyObject* py_mean_filter(PyObject* self, PyObject* args) {
+    PyArrayObject* array;
+    PyArrayObject* Bc;
+    PyArrayObject* output;
+    int mode;
+    double cval;
+    if (!PyArg_ParseTuple(args, "OOOid", &array, &Bc, &output, &mode, &cval) ||
+        !numpy::are_arrays(array, Bc, output) ||
+        !numpy::equiv_typenums(array, Bc) ||
+        !numpy::check_type<double>(output) ||
+        !PyArray_ISCARRAY(output)) {
+        PyErr_SetString(PyExc_RuntimeError,TypeErrorMsg);
+        return NULL;
+    }
+    holdref r(output);
+
+#define HANDLE(type) \
+        mean_filter<type>(numpy::aligned_array<double>(output), numpy::aligned_array<type>(array), numpy::aligned_array<type>(Bc), mode, cval);
+    SAFE_SWITCH_ON_TYPES_OF(array);
+#undef HANDLE
+
+    Py_INCREF(output);
+    return PyArray_Return(output);
+}
+
 template <typename T>
-void template_match(numpy::aligned_array<T> res, const numpy::aligned_array<T> f, const numpy::aligned_array<T> t, int mode) {
+void template_match(numpy::aligned_array<T> res, const numpy::aligned_array<T> f, const numpy::aligned_array<T> t, int mode, bool just_equality) {
     gil_release nogil;
     const int N = res.size();
     typename numpy::aligned_array<T>::const_iterator iter = f.begin();
@@ -568,6 +614,10 @@ void template_match(numpy::aligned_array<T> res, const numpy::aligned_array<T> f
             if (fiter.retrieve(iter, j, val)) {
                 const T tj = fiter[j];
                 const T delta = (val > tj ? val - tj : tj - val);
+                if (just_equality && delta) {
+                    diff2 = 1;
+                    break;
+                }
                 diff2 += delta*delta;
             }
         }
@@ -579,8 +629,9 @@ PyObject* py_template_match(PyObject* self, PyObject* args) {
     PyArrayObject* array;
     PyArrayObject* template_;
     int mode;
+    int just_equality;
     PyArrayObject* output;
-    if (!PyArg_ParseTuple(args, "OOOi", &array, &template_, &output, &mode)) return NULL;
+    if (!PyArg_ParseTuple(args, "OOOii", &array, &template_, &output, &mode, &just_equality)) return NULL;
     if (!numpy::are_arrays(array, template_, output) ||
         !numpy::equiv_typenums(array, template_, output) ||
         !PyArray_ISCARRAY(output)) {
@@ -590,7 +641,7 @@ PyObject* py_template_match(PyObject* self, PyObject* args) {
     holdref r(output);
 
 #define HANDLE(type) \
-        template_match<type>(numpy::aligned_array<type>(output), numpy::aligned_array<type>(array), numpy::aligned_array<type>(template_), mode);
+        template_match<type>(numpy::aligned_array<type>(output), numpy::aligned_array<type>(array), numpy::aligned_array<type>(template_), mode, just_equality);
     SAFE_SWITCH_ON_TYPES_OF(array);
 #undef HANDLE
 
@@ -598,6 +649,60 @@ PyObject* py_template_match(PyObject* self, PyObject* args) {
     return PyArray_Return(output);
 }
 
+template <typename T>
+void find2d(const numpy::aligned_array<T> array, const numpy::aligned_array<T> target, numpy::aligned_array<bool> out) {
+    gil_release nogil;
+    const int N0 = array.dim(0);
+    const int N1 = array.dim(1);
+
+    const int Nt0 = target.dim(0);
+    const int Nt1 = target.dim(1);
+    assert(out.is_carray());
+    bool* rpos = out.data();
+    std::fill(rpos, rpos + N0*N1, false);
+
+    for (int y = 0; y < N0 - Nt0; ++y) {
+        for (int x = 0; x < N1 - Nt1; ++x) {
+            for (int sy = 0; sy < Nt0; ++sy) {
+                for (int sx = 0; sx < Nt1; ++sx) {
+                    if (array.at(y + sy,x + sx) != target.at(sy,sx)) {
+                        goto next_pos;
+                    }
+                }
+            }
+            out.at(y, x) = true;
+            next_pos:
+                ;
+        }
+    }
+}
+
+PyObject* py_find2d(PyObject* self, PyObject* args) {
+    PyArrayObject* array;
+    PyArrayObject* target;
+    PyArrayObject* output;
+    if (!PyArg_ParseTuple(args,"OOO", &array, &target, &output)) return NULL;
+    if (!numpy::are_arrays(array, target, output) ||
+            !numpy::same_shape(output, array) ||
+            !numpy::equiv_typenums(array, target) ||
+            !numpy::check_type<bool>(output) ||
+            !PyArray_ISCARRAY(output)) {
+            PyErr_SetString(PyExc_RuntimeError, OutputErrorMsg);
+            return NULL;
+    }
+    holdref outref(output);
+
+#define HANDLE(type) \
+    find2d<type>(numpy::aligned_array<type>(array), numpy::aligned_array<type>(target), numpy::aligned_array<bool>(output));
+
+    SAFE_SWITCH_ON_TYPES_OF(array);
+#undef HANDLE
+
+    Py_INCREF(output);
+    return PyArray_Return(output);
+}
+
+
 PyMethodDef methods[] = {
   {"convolve",(PyCFunction)py_convolve, METH_VARARGS, NULL},
   {"convolve1d",(PyCFunction)py_convolve1d, METH_VARARGS, NULL},
@@ -608,7 +713,9 @@ PyMethodDef methods[] = {
   {"haar",(PyCFunction)py_haar, METH_VARARGS, NULL},
   {"ihaar",(PyCFunction)py_ihaar, METH_VARARGS, NULL},
   {"rank_filter",(PyCFunction)py_rank_filter, METH_VARARGS, NULL},
+  {"mean_filter",(PyCFunction)py_mean_filter, METH_VARARGS, NULL},
   {"template_match",(PyCFunction)py_template_match, METH_VARARGS, NULL},
+  {"find2d",(PyCFunction)py_find2d, METH_VARARGS, NULL},
   {NULL, NULL,0,NULL},
 };
 
