@@ -1,4 +1,4 @@
-/* Copyright (C) 2010-2013  Luis Pedro Coelho <luis@luispedro.org>
+/* Copyright (C) 2010-2014  Luis Pedro Coelho <luis@luispedro.org>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -33,6 +33,7 @@
 #include <cmath>
 #include <vector>
 #include <map>
+#include <queue>
 #include <functional>
 
 #include "numpypp/array.hpp"
@@ -40,6 +41,7 @@
 #include "numpypp/dispatch.hpp"
 #include "utils.hpp"
 #include "_filters.h"
+#include "debug.h"
 
 namespace{
 
@@ -49,18 +51,20 @@ const char TypeErrorMsg[] =
 
 
 // This is a standard union-find structure
-int find(int* data, int i) {
+template<typename It>
+int find(It data, int i) {
     if (data[i] == i) return i;
     int j = find(data, data[i]);
     data[i] = j;
     return j;
 }
-void compress(int* data, int i) {
+template<typename It>
+void compress(It data, int i) {
     find(data,i);
 }
 
-
-void join(int* data, int i, int j) {
+template<typename It>
+void join(It data, int i, int j) {
     i = find(data, i);
     j = find(data, j);
     assert(i >= 0);
@@ -71,7 +75,7 @@ void join(int* data, int i, int j) {
 int label(numpy::aligned_array<int> labeled, const numpy::aligned_array<int> Bc) {
     gil_release nogil;
     const int N = labeled.size();
-    int* data = labeled.data();
+    get_pointer_type<int>::ptr data = as_checked_ptr(labeled.data(), N);
     for (int i = 0; i != N; ++i) {
         data[i] = (data[i] ? i : -1);
     }
@@ -191,7 +195,7 @@ bool border(const numpy::aligned_array<T> array, const numpy::aligned_array<T> f
     gil_release nogil;
     const int N = array.size();
     typename numpy::aligned_array<T>::const_iterator iter = array.begin();
-    filter_iterator<T> fiter(array.raw_array(), filter.raw_array(), EXTEND_CONSTANT, true);
+    filter_iterator<T> fiter(array.raw_array(), filter.raw_array(), ExtendConstant, true);
     const int N2 = fiter.size();
     bool* out = result.data();
     bool any = false;
@@ -480,6 +484,49 @@ struct centroid_info {
     float x;
 };
 
+bool fetch_neighbour(int n, int y, int x, int& ny, int& nx, const int Ny, const int Nx) {
+    ny = y;
+    nx = x;
+    switch(n) {
+        case 0:
+            ++nx;
+            if (nx < Nx) return true;
+            return false;
+        case 1:
+            --nx;
+            if (nx >= 0) return true;
+            return false;
+        case 2:
+            ++ny;
+            if (ny < Ny) return true;
+            return false;
+        case 3:
+            --ny;
+            if (ny >= 0) return true;
+            return false;
+        default:
+            assert(0);
+    }
+}
+
+struct SlicPoint {
+    SlicPoint(int y, int x, int ci, int cost)
+        :y(y)
+        ,x(x)
+        ,ci(ci)
+        ,cost(cost)
+        { }
+
+    bool operator < (const SlicPoint& other) const { return this->cost > other.cost; }
+
+    int y;
+    int x;
+    int ci;
+    int cost;
+};
+
+int diff2(int a, int b) { return (a-b)*(a-b); }
+
 int slic(const numpy::aligned_array<npy_float32> array, numpy::aligned_array<int> alabels, const int S, const float m, const int max_iters) {
     assert(alabels.is_carray());
     gil_release no_gil;
@@ -530,11 +577,8 @@ int slic(const numpy::aligned_array<npy_float32> array, numpy::aligned_array<int
                     float l = array.at(y,x,0);
                     float a = array.at(y,x,1);
                     float b = array.at(y,x,2);
-                    const float Ds = (c.y - y)*(c.y - y) +
-                            (c.x - x)*(c.x - x);
-                    const float Dc = (c.l - l)*(c.l - l) +
-                            (c.a - a)*(c.a - a) +
-                            (c.b - b)*(c.b - b);
+                    const float Ds = diff2(y, c.y) + diff2(x, c.x);
+                    const float Dc = diff2(l, c.l) + diff2(a, c.a) + diff2(b, c.b);
                     const float D2 = Dc + Ds*m2S2;
 
                     assert(D2 < inf);
@@ -589,42 +633,55 @@ int slic(const numpy::aligned_array<npy_float32> array, numpy::aligned_array<int
     }
 
     for (int i = 0; i != N; ++i) nlabels[i] = i;
-    int * nlabelsp = &nlabels[0];
+    std::vector<int>::iterator nlabelsp = nlabels.begin();
 
     for (int y = 0; y != Ny; ++y) {
         for (int x = 0; x != Nx; ++x) {
-#define CHECK(dy,dx) \
-            if ((y + dy) >= 0 && (y + dy) < alabels.dim(0) && (x + dx) >= 0 && (x + dx) < alabels.dim(1)) { \
-                if (alabels.at(y,x) == alabels.at(y+dy,x + dx)) { \
-                    const int i = y*Nx + x; \
-                    const int j = (y+dy)*Nx + x + dx; \
-                    join(nlabelsp, i, j); \
-                } \
+            int nx, ny;
+            for (int n = 0; n != 4; ++n) {
+                if (fetch_neighbour(n, y, x, ny, nx, Ny, Nx) &&
+                    (alabels.at(y,x) == alabels.at(ny,nx))) {
+                    const int i =  y*Nx +  x;
+                    const int j = ny*Nx + nx;
+                    join(nlabelsp, i, j);
+                }
             }
-            CHECK(-1,0)
-            CHECK(0,1)
-            CHECK(0,-1)
-            CHECK(1,0)
         }
     }
+    std::vector<bool> is_connected(N);
     for (int y = 0; y != Ny; ++y) {
         for (int x = 0; x != Nx; ++x) {
             const int i = y*Nx + x;
             const int cy = centroids[alabels.at(y,x)].y;
             const int cx = centroids[alabels.at(y,x)].x;
             const int j = cy*Nx + cx;
-            if (find(nlabelsp, i) != find(nlabelsp, j)) {
-                float min_d = inf;
-                int k = -1;
-                for (unsigned ci = 0; ci < centroids.size(); ++ci) {
-                    const centroid_info& c = centroids[ci];
-                    const float d = (c.y - y)*(c.y - y) + (c.x - x)*(c.x - x);
-                    if (d < min_d) {
-                        min_d = d;
-                        k = ci;
-                    }
+            is_connected[i] = (find(nlabelsp, i) == find(nlabelsp, j));
+        }
+    }
+    std::vector<bool> seen(N);
+    std::priority_queue<SlicPoint> q;
+    for (unsigned ci = 0; ci < centroids.size(); ++ci) {
+        const centroid_info& c = centroids[ci];
+        q.push(SlicPoint(c.y, c.x, ci, 0));
+    }
+    while (!q.empty()) {
+        SlicPoint p = q.top();
+        q.pop();
+        const int i = p.y*Nx + p.x;
+        const centroid_info& c = centroids[p.ci];
+        if (seen[i]) continue;
+        seen[i] = true;
+        if (!is_connected[i]) {
+            alabels.at(p.y,p.x) = p.ci;
+        }
+        for (int n = 0; n != 4; ++n) {
+            int ny, nx;
+            if (fetch_neighbour(n, p.y, p.x, ny, nx, Ny, Nx)) {
+                int j = ny*Nx + nx;
+                if (!seen[j] && (!is_connected[j] || alabels.at(ny,nx) == p.ci)) {
+                    const int cost = diff2(ny, c.y) + diff2(nx, c.x);
+                    q.push(SlicPoint(ny, nx, p.ci, cost));
                 }
-                alabels.at(y,x) = k;
             }
         }
     }
