@@ -15,6 +15,7 @@ __all__ = [
     'convolve1d',
     'daubechies',
     'idaubechies',
+    'find',
     'haar',
     'ihaar',
     'median_filter',
@@ -157,6 +158,41 @@ def median_filter(f, Bc=None, mode='reflect', cval=0.0, out=None, output=None):
     _check_mode(mode, cval, 'median_filter')
     return _convolve.rank_filter(f, Bc, output, int(rank), mode2int[mode])
 
+def mean_filter(f, Bc, mode='ignore', cval=0.0, out=None):
+    '''mean = mean_filter(f, Bc, mode='ignore', cval=0.0, out=None)
+
+    Mean filter. The value at ``mean[i,j]`` will be the mean of the values in
+    the neighbourhood defined by ``Bc``.
+
+    Parameters
+    ----------
+    f : ndarray
+        input. Any dimension is supported
+    Bc : ndarray
+        Defines the neighbourhood. Must be explicitly passed, no default.
+    mode : {'reflect', 'nearest', 'wrap', 'mirror', 'constant', 'ignore' [ignore]}
+        How to handle borders. The default is to ignore points beyond the
+        border, so that the means computed near the border include fewer elements.
+    cval : double, optional
+        If `mode` is constant, which constant to use (default: 0.0)
+    out : ndarray, optional
+        Output array. Must be a double array with the same shape as `f` as well
+        as be C-contiguous.
+
+    Returns
+    -------
+    mean : ndarray of type double and same shape as ``f``
+
+    See Also
+    --------
+    median_filter : An alternative filtering method
+    '''
+    Bc = morph.get_structuring_elem(f, Bc)
+    out = _get_output(f, out, 'mean_filter', dtype=np.float64)
+    _check_mode(mode, cval, 'mean_filter')
+    return _convolve.mean_filter(f, Bc, out, mode2int[mode], cval)
+
+
 def rank_filter(f, Bc, rank, mode='reflect', cval=0.0, out=None, output=None):
     '''
     ranked = rank_filter(f, Bc, rank, mode='reflect', cval=0.0, out=None)
@@ -195,15 +231,17 @@ def rank_filter(f, Bc, rank, mode='reflect', cval=0.0, out=None, output=None):
 
 
 def template_match(f, template, mode='reflect', cval=0., out=None, output=None):
-    '''
-    match = template_match(f, template, mode='reflect', cval=0., out={np.empty_like(f)})
+    '''Match template to image
 
-    Match template.
+    match = template_match(f, template, mode='reflect', cval=0., out={np.empty_like(f)})
 
     The value at ``match[i,j]`` will be the difference (in squared euclidean
     terms), between `template` and a same sized window on `f` centered on that
     point.
 
+    Note that the computation is performed using the same dtype as ``f``. Thus
+    is may overflow if the template is large.
+
     Parameters
     ----------
     f : ndarray
@@ -228,7 +266,35 @@ def template_match(f, template, mode='reflect', cval=0., out=None, output=None):
     template = template.astype(f.dtype)
     output = _get_output(f, out, 'template_match', output=output)
     _check_mode(mode, cval, 'template_match')
-    return _convolve.template_match(f, template, output, mode2int[mode])
+    return _convolve.template_match(f, template, output, mode2int[mode], 0)
+
+def find(f, template):
+    '''Match template to image exactly
+
+    coordinates = find(f, template)
+
+    The output is in the same format as the ``np.where`` function.
+
+    Parameters
+    ----------
+    f : ndarray
+        input. Currently, only 2-dimensional images are supported.
+    template : ndarray
+        Template to match. Must be explicitly passed, no default.
+
+    Returns
+    -------
+    match : np.array
+    coordinates : np.array
+        These are the coordinates of the match. The format is similar to the
+        output of ``np.where``, but in an ndarray.
+
+    '''
+    if f.ndim != 2:
+        raise ValueError('mahotas.find: Cannot handle multi-dimensional images')
+    template = template.astype(f.dtype)
+    out = np.empty(f.shape, bool)
+    return _convolve.find2d(f, template, out)
 
 
 def gaussian_filter1d(array, sigma, axis=-1, order=0, mode='reflect', cval=0., out=None, output=None):
