@@ -288,7 +288,8 @@ def cwatershed(surface, markers, Bc=None, return_lines=False):
     ----------
     surface : image
     markers : image
-        initial markers (must be a labeled image)
+        initial markers (must be a labeled image, i.e., one where 0 represents
+        the background and higher integers represent different regions)
     Bc : ndarray, optional
         structuring element (default: 3x3 cross)
     return_lines : boolean, optional
@@ -296,15 +297,15 @@ def cwatershed(surface, markers, Bc=None, return_lines=False):
 
     Returns
     -------
-    W : Regions image (i.e., W[i,j] == region for pixel (i,j))
+    W : integer ndarray (int64 ints)
+        Regions image (i.e., W[i,j] == region for pixel (i,j))
     WL : Lines image (`if return_lines==True`)
     '''
     _verify_is_integer_type(surface, 'cwatershed')
     _verify_is_integer_type(markers, 'cwatershed')
+    markers = markers.astype(np.int64)
     if surface.shape != markers.shape:
         raise ValueError('morph.cwatershed: Markers array should have the same shape as value array.')
-    if markers.dtype != surface.dtype:
-        markers = markers.astype(surface.dtype)
     Bc = get_structuring_elem(surface, Bc)
     return _morph.cwatershed(surface, markers, Bc, bool(return_lines))
 
